@@ -39,11 +39,11 @@ public:
   Base_DB();
   virtual ~Base_DB();
 
-  /// Specify the structure of what will be shown.
+  /// Specify the structure of what will be shown, and fill it.
   virtual void init_db_details();
 
   /// Specify what actual data will be shown:
-  virtual void refresh_db_details();
+  virtual void refresh_data_from_database();
 
   /** Returns whether we are in developer mode.
    * Some functionality will be deactivated when not in developer mode.
