@@ -549,14 +549,14 @@ void Frame_Glom::update_table_in_document_from_database()
       else //if it was found.
       {
         //Compare the information:
-        Gnome::Gda::FieldAttributes field_info_db =  field_database.get_field_info();
+        Gnome::Gda::FieldAttributes field_info_db = field_database.get_field_info();
         Field field_document =  *iterFindDoc;
         if(field_document.field_info_from_database_is_equal( field_info_db )) //ignores auto_increment because libgda does not report it from the database properly.
         {
           //The database has different information. We assume that the information in the database is newer.
 
           //Update the field information:
-          field_info_db.set_auto_increment( field_document.get_field_info().get_auto_increment() ); //libgda does not report it from the database properly.
+          field_info_db.set_auto_increment( field_document.get_auto_increment() ); //libgda does not report it from the database properly.
           iterFindDoc->set_field_info( field_info_db );
 
           document_must_to_be_updated = true;
@@ -592,13 +592,14 @@ void Frame_Glom::set_document(Document_Glom* pDocument)
 {
   View_Composite_Glom::set_document(pDocument);
 
-  if(m_pDocument)
+  Document_Glom* document = get_document();
+  if(document)
   {
     //Connect to a signal that is only on the derived document class:
-    m_pDocument->signal_userlevel_changed().connect( sigc::mem_fun(*this, &Frame_Glom::on_userlevel_changed) );
+    document->signal_userlevel_changed().connect( sigc::mem_fun(*this, &Frame_Glom::on_userlevel_changed) );
 
     //Show the appropriate UI for the user level that is specified by this new document:
-    on_userlevel_changed(m_pDocument->get_userlevel());
+    on_userlevel_changed(document->get_userlevel());
   }
 }
 
@@ -662,6 +663,7 @@ void Frame_Glom::on_menu_developer_users()
   dialog->load_from_document(); //Update the UI now that it has the document.
 
   dialog->run();
+  remove_view(dialog);
   delete dialog;
 
   //Update the Details and List layouts, in case the permissions have changed:
