@@ -53,6 +53,7 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
     if(dialog)
     {
       m_pDialogLayout = dialog;
+      add_view(m_pDialogLayout); //Give it access to the document.
       m_pDialogLayout->signal_hide().connect( sigc::mem_fun(static_cast<Box_Data&>(*this), &Box_Data::on_dialog_layout_hide) );
     }
   }
@@ -134,23 +135,26 @@ void Box_Data_Details::init_db_details(const Glib::ustring& strTableName, const
 
   get_field_primary_key_for_table(strTableName, m_field_primary_key);
 
-  Box_Data::init_db_details(strTableName);
+  Box_Data::init_db_details(strTableName); //Calls create_layout(), then fill_from_database()
 }
 
-void Box_Data_Details::refresh_db_details(const Gnome::Gda::Value& primary_key_value)
+void Box_Data_Details::refresh_data_from_database(const Gnome::Gda::Value& primary_key_value)
 {
-  init_db_details(get_table_name(), primary_key_value);
+  m_primary_key_value = primary_key_value;
+  fill_from_database();
 }
 
-void Box_Data_Details::refresh_db_details_blank()
+void Box_Data_Details::refresh_data_from_database_blank()
 {
-  refresh_db_details( Gnome::Gda::Value() );
+  refresh_data_from_database( Gnome::Gda::Value() );
 }
 
-void Box_Data_Details::fill_from_database_layout()
+void Box_Data_Details::create_layout()
 {
   Bakery::BusyCursor(*get_app_window());
 
+  Box_Data::create_layout(); //Fills m_TableFields.
+
   //Remove existing child widgets:
   m_FlowTable.remove_all();
 
@@ -162,7 +166,6 @@ void Box_Data_Details::fill_from_database_layout()
     //This map of layout groups will also contain the field information from the database:
     Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups("details");
 
-
     for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
     {
       m_FlowTable.add_layout_group(iter->second);
@@ -182,15 +185,13 @@ void Box_Data_Details::fill_from_database()
     sharedptr<SharedConnection> sharedconnection = connect_to_server();
 
 
-    Box_DB_Table::fill_from_database();
+    Box_Data::fill_from_database();
 
-    m_Fields = get_fields_to_show();
-    type_vecLayoutFields fieldsToGet = m_Fields;
+    m_FieldsShown = get_fields_to_show();
+    type_vecLayoutFields fieldsToGet = m_FieldsShown;
 
     if(!fieldsToGet.empty())
     {
-      fill_from_database_layout(); //TODO: Only do this when the layout has changed.
-
       //Do not try to show the data if the user may not view it:
       Privileges table_privs = get_current_privs(m_strTableName);
 
@@ -253,111 +254,23 @@ void Box_Data_Details::fill_from_database()
   }
 }
 
-/*
-void Box_Data_Details::fill_related()
-{
-  if(!m_bDoNotRefreshRelated)
-  {
-    //Clear existing pages:
-    {
-      Gtk::Notebook_Helpers::PageList& pages = m_Notebook_Related.pages();
-      for(Gtk::Notebook_Helpers::PageList::iterator iter = pages.begin(); iter != pages.end(); iter++)
-      {
-        Gtk::Notebook_Helpers::Page page = *iter;
-        Box_Data_List_Related* pWidget = dynamic_cast<Box_Data_List_Related*>(page.get_child());
-        if(pWidget)
-          remove_view(pWidget);
-        else
-          std::cout << "Box_Data_Details::fill_related(): unexpected child widget." << std::endl;
-      }
-
-      pages.clear(); //deletes Gtk::manage()d boxes.
-    }
-
-    //Get relationships from the document:
-    Document_Glom::type_vecRelationships vecRelationships = m_pDocument->get_relationships(m_strTableName);
-
-    if(vecRelationships.empty())
-    {
-      //Hide the relationships pane:
-      m_Frame_Related.hide();
-    }
-    else
-    {
-      m_Frame_Related.show();
-      
-      //Add the relationships:
-      for(Document_Glom::type_vecRelationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); iter++)
-      {
-       const Relationship& relationship = *iter;
-
-       //bool bMakeRelatedTab = true;
-       //guint rowKey = 0;
-
-       Glib::ustring from_field = relationship.get_from_field();
-       if(!from_field.empty())
-       {
-         const Glib::ustring to_table_name =  relationship.get_to_table();
-         const Glib::ustring to_field_name = relationship.get_to_field();
-
-         //Check that the to_field still exists in the to_table structure:
-         //(note that the to_field will not necessarily be visible in the related records list, though it must be in the database
-         //TODO_performance:
-         type_vecFields vecRelatedFields = get_fields_for_table_from_database(to_table_name); //Fields in the database;
-         type_vecFields::const_iterator iterFind = std::find_if(vecRelatedFields.begin(), vecRelatedFields.end(), predicate_FieldHasName<Field>(to_field_name));
-         
-         if(iterFind != vecRelatedFields.end() ) //If it was found
-         {
-           Box_Data_List_Related* pBox = Gtk::manage(new Box_Data_List_Related());
-           add_view(pBox); //So that it knows about the Document.
-           m_Notebook_Related.pages().push_back( Gtk::Notebook_Helpers::TabElem(*pBox, relationship.get_name()) );
-              
-           Gnome::Gda::Value value = m_FlowTable.get_field_value(from_field);
-
-           Field field;
-           bool test = get_fields_for_table_one_field(m_strTableName, from_field, field);
-           if(test)
-           {
-             pBox->init_db_details(relationship, value, m_primary_key_value);
-             pBox->show_all();
-
-
-             //Connect signals:
-             pBox->signal_record_added.connect( sigc::bind(
-               sigc::mem_fun(*this, &Box_Data_Details::on_related_record_added), relationship.get_from_field() )
-             );
-
-             pBox->signal_user_requested_details().connect( sigc::bind(
-               sigc::mem_fun(*this, &Box_Data_Details::on_related_user_requested_details), relationship.get_to_table() )
-             );
-           }
-         }
-       }
-      }
-    }
-  }
-
-}
-*/
-
 void Box_Data_Details::on_button_new()
 {
   if(confirm_discard_unstored_data())
   {
-    Gnome::Gda::FieldAttributes fieldinfo = m_field_primary_key.get_field_info();
-    if(fieldinfo.get_auto_increment()) //If the primary key is an auto-increment:
+    if(m_field_primary_key.get_auto_increment()) //If the primary key is an auto-increment:
     {
       //Just make a new record, and show it:
       Gnome::Gda::Value primary_key_value = generate_next_auto_increment(m_strTableName, m_field_primary_key.get_name()); //TODO: This should return a Gda::Value
 
       record_new(false /* use entered field data */, primary_key_value);
-      refresh_db_details(primary_key_value);
+      refresh_data_from_database(primary_key_value);
     }
     else
     {
       //It's not an auto-increment primary key,
       //so just blank the fields ready for a primary key later.
-      refresh_db_details_blank(); //shows blank record.
+      refresh_data_from_database_blank(); //shows blank record.
     }
 
   } //if(confirm_discard_unstored_data())
@@ -443,31 +356,32 @@ void Box_Data_Details::on_related_user_requested_details(Gnome::Gda::Value key_v
 
 void Box_Data_Details::recalculate_fields_for_related_records(const Glib::ustring& relationship_name)
 {
- //TODO: Get (recursively) the list of fields to recalcualte, sort them in order of dependency, then calculate them all.
+  m_FieldsCalculationInProgress.clear();
 
   const Gnome::Gda::Value primary_key_value = get_primary_key_value();
 
-  for(type_vecLayoutFields::iterator iter = m_Fields.begin(); iter != m_Fields.end(); ++iter)
+  for(type_vecFields::iterator iter = m_TableFields.begin(); iter != m_TableFields.end(); ++iter)
   {
-    const LayoutItem_Field& field = *iter;
+    const Field& field = *iter;
 
     //Is this field triggered by this relationship?
-    const Field::type_list_strings triggered_by = field.m_field.get_calculation_relationships();
+    const Field::type_list_strings triggered_by = field.get_calculation_relationships();
     Field::type_list_strings::const_iterator iterFind = std::find(triggered_by.begin(), triggered_by.end(), relationship_name);
     if(iterFind != triggered_by.end()) //If it was found
     {
-       //recalculate:
-       const type_map_fields field_values = get_record_field_values(primary_key_value);
-       Gnome::Gda::Value value = glom_evaluate_python_function_implementation(field.m_field.get_glom_type(), iter->m_field.get_calculation(), field_values,
-          get_document(), m_strTableName);
+      const Field& field = *iter;
 
-      //show it:
-      set_entered_field_data(field, value);
+      calculate_field(field, m_field_primary_key, primary_key_value); //And any dependencies.
 
-      //Add it to the database (even if it is not shown in the view)
-      set_field_value_in_database(field, value, m_field_primary_key, primary_key_value);
+      //Calculate anything that depends on this.
+      LayoutItem_Field layout_item;
+      layout_item.m_field = field;
+
+      do_calculations(layout_item, m_field_primary_key, primary_key_value, false /* recurse, reusing m_FieldsCalculationInProgress */);
     }
   }
+
+   m_FieldsCalculationInProgress.clear();
 }
 
 void Box_Data_Details::on_related_record_added(Gnome::Gda::Value /* strKeyValue */, Glib::ustring /* strFromKeyName */)
@@ -543,7 +457,7 @@ void Box_Data_Details::on_flowtable_layout_changed()
     document->set_data_layout_groups("details", m_strTableName, layout_groups);
 
   //Build the view again from the new layout:
-  fill_from_database_layout();
+  create_layout();
 
   //And fill it with data:
   fill_from_database();
@@ -640,7 +554,7 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
         //If this is a foreign key then refresh the related records:
         /*
         bool bIsForeignKey = false;
-        Document_Glom::type_vecRelationships vecRelationships = m_pDocument->get_relationships(m_strTableName);
+        Document_Glom::type_vecRelationships vecRelationships = get_document()->get_relationships(m_strTableName);
         for(Document_Glom::type_vecRelationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); iter++)
         {
           const Relationship& relationship = *iter;
@@ -667,7 +581,7 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
   {
     //There is no current primary key value:
 
-    if(m_field_primary_key.get_field_info().get_auto_increment()) //If the primary key is an auto-increment:
+    if(m_field_primary_key.get_auto_increment()) //If the primary key is an auto-increment:
     {
       if(strFieldName == m_field_primary_key.get_name()) //If edited field is the primary key.
       {
