@@ -43,7 +43,7 @@ public:
    * @param foreign_key_value: The value that should be found in this table.
    * @param from_table_primary_key_value The primary key of the parent record's table, used to associate new related records.
    */
-  virtual void refresh_db_details(const Gnome::Gda::Value& foreign_key_value, const Gnome::Gda::Value& from_table_primary_key_value);
+  virtual void refresh_data_from_database(const Gnome::Gda::Value& foreign_key_value);
 
   virtual Relationship get_relationship() const;
   virtual Field get_key_field() const;
@@ -52,12 +52,18 @@ public:
 
   sigc::signal<void, Gnome::Gda::Value> signal_record_added;
 
+  ///@param relationship_name
+  typedef sigc::signal<void, const Glib::ustring&> type_signal_record_changed;
+  type_signal_record_changed signal_record_changed();
+
 protected:
   virtual void fill_from_database(); //Override.
   virtual type_vecLayoutFields get_fields_to_show() const; //override
 
+  virtual void on_adddel_user_changed(const Gtk::TreeModel::iterator& row, guint col);
   virtual void on_adddel_user_added(const Gtk::TreeModel::iterator& row, guint col_with_first_value); //Override.
   virtual void on_record_added(const Gnome::Gda::Value& primary_key_value); //Override. Not a signal handler.
+  virtual void on_record_deleted(const Gnome::Gda::Value& primary_key_value); //override.
   virtual void on_dialog_layout_hide(); //override.
 
   virtual void enable_buttons();
@@ -71,6 +77,8 @@ protected:
   Relationship m_relationship; //The relationship of the parent table to this one.
   Field m_key_field;
   Gnome::Gda::Value m_key_value;
+
+  type_signal_record_changed m_signal_record_changed;
 };
 
 #endif
