@@ -48,9 +48,9 @@ void Box_Data_List_Find::fill_from_database()
   m_AddDel.remove_all();
 
   //Field Names:
-  fill_column_titles();
+  create_layout();
 
-  m_Fields = get_fields_to_show();
+  m_FieldsShown = get_fields_to_show();
 
   m_AddDel.add_item(Gnome::Gda::Value("find"));
 }
