@@ -21,11 +21,12 @@
 #include "db_adddel.h"
 #include <algorithm> //For std::find.
 #include <glibmm/i18n.h>
-#include "../adddel/cellrendererlist.h"
+#include "../cellrendererlist.h"
 #include "db_treeviewcolumn_glom.h"
 #include "../../data_structure/glomconversions.h"
 #include "../../dialog_invalid_data.h"
 #include "../../application.h"
+//#include "../cellrendererlist.h"
 #include <iostream> //For debug output.
 
 DbAddDelColumnInfo::DbAddDelColumnInfo()
@@ -580,21 +581,6 @@ void DbAddDel::construct_specified_columns()
 
   Gtk::TreeModel::ColumnRecord record;
 
-  //Hidden internal columns:
-
-  //We must recreate these standard modelcolumns, because we can not reuse them
-  //if(m_modelcolumn_key)
-  //  delete m_modelcolumn_key;
-
-  //m_modelcolumn_key = new Gtk::TreeModelColumn<Gnome::Gda::Value>;
-
-  //if(m_modelcolumn_placeholder)
-  //  delete m_modelcolumn_placeholder;
-  //m_modelcolumn_placeholder = new Gtk::TreeModelColumn<bool>;
-
-  //record.add(*m_modelcolumn_key);
-  //record.add(*m_modelcolumn_placeholder);
-
   //Database columns:
   type_model_store::type_vec_fields fields;
   {
@@ -619,7 +605,7 @@ void DbAddDel::construct_specified_columns()
   bool key_found = false;
   for(type_model_store::type_vec_fields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
   {
-    if( !(iter->get_has_relationship_name()) && (iter->m_field.get_field_info().get_primary_key()) )
+    if( !(iter->get_has_relationship_name()) && (iter->m_field.get_primary_key()) )
     {
       key_found = true;
       break;
@@ -665,35 +651,6 @@ void DbAddDel::construct_specified_columns()
       Gtk::CellRenderer* pCellRenderer = 0;
       switch(column_info.m_field.m_field.get_glom_type())
       {
-  /*
-        case(DbAddDelColumnInfo::STYLE_Choices):
-        {
-          //Use a custom CellRenderer:
-          CellRendererList* pCellRendererDerived = Gtk::manage( new CellRendererList() );
-
-          //Add the choices:
-          const type_vecStrings vecStrings = column_info.m_choices;
-          for(type_vecStrings::const_iterator iter = vecStrings.begin(); iter != vecStrings.end(); ++iter)
-          {
-            pCellRendererDerived->append_list_item(*iter);
-          }
-
-          pCellRenderer = pCellRendererDerived;
-
-          break;
-        }
-
-   */
-
-    /* Possible types:
-    TYPE_INVALID,
-    TYPE_NUMERIC,
-    TYPE_TEXT,
-    TYPE_DATE,
-    TYPE_TIME,
-    TYPE_BOOLEAN
-    */
-
         case(Field::TYPE_BOOLEAN):
         {
           pCellRenderer = Gtk::manage( new Gtk::CellRendererToggle() );
@@ -702,7 +659,15 @@ void DbAddDel::construct_specified_columns()
         }
         default:
         {
-          pCellRenderer = Gtk::manage( new Gtk::CellRendererText() );
+          if(column_info.m_field.get_has_choices())
+          {
+            CellRendererList* rendererList = Gtk::manage( new CellRendererList() );
+            rendererList->set_restrict_values_to_list(column_info.m_field.get_choices_restricted());
+
+            pCellRenderer = rendererList;
+          }
+          else
+            pCellRenderer = Gtk::manage( new Gtk::CellRendererText() );
 
           break;
         }
@@ -718,15 +683,73 @@ void DbAddDel::construct_specified_columns()
         Gtk::CellRendererText* pCellRenderer = dynamic_cast<Gtk::CellRendererText*>(m_TreeView.get_column_cell_renderer(view_column_index));
         if(pCellRenderer)
         {
-          //Connect a signal handler:
-          if(pCellRenderer)
-          {
-            //Make it editable:
-            pCellRenderer->property_editable() = true;
+          //Make it editable:
+          pCellRenderer->property_editable() = true;
 
-            //Connect to its signal:
-            pCellRenderer->signal_edited().connect(
-              sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_treeview_cell_edited), model_column_index) );
+          if( column_info.m_field.m_field.get_glom_type() == Field::TYPE_NUMERIC )
+            pCellRenderer->property_xalign() = 1.0; //Align right.
+
+          //Connect to its signal:
+          pCellRenderer->signal_edited().connect(
+            sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_treeview_cell_edited), model_column_index) );
+
+          CellRendererList* pCellRendererCombo = dynamic_cast<CellRendererList*>(pCellRenderer);
+          if(pCellRendererCombo)
+          {
+            pCellRendererCombo->remove_all_list_items();
+
+            if(column_info.m_field.get_has_custom_choices())
+            {
+              pCellRendererCombo->set_use_second(false); //Custom choices have only one column.
+
+              //set_choices() needs this, for the numeric layout:
+              //pCellRendererCombo->set_layout_item(get_layout_item()->clone(), table_name); //TODO_Performance: We only need this for the numerical format.
+              const LayoutItem_Field::type_list_values list_values = column_info.m_field.get_choices_custom();
+              for(LayoutItem_Field::type_list_values::const_iterator iter = list_values.begin(); iter != list_values.end(); ++iter)
+              {
+                pCellRendererCombo->append_list_item( GlomConversions::get_text_for_gda_value(column_info.m_field.m_field.get_glom_type(), *iter, column_info.m_field.m_numeric_format) );
+              }
+            }
+            else if(column_info.m_field.get_has_related_choices())
+            {
+              Glib::ustring choice_relationship_name, choice_field, choice_second;
+              column_info.m_field.get_choices(choice_relationship_name, choice_field, choice_second);
+              if(!choice_relationship_name.empty() && !choice_field.empty())
+              {
+                const Relationship relationship = column_info.m_field.m_choices_related_relationship;
+                const Glib::ustring to_table = relationship.get_to_table();
+
+                const bool use_second = !choice_second.empty();
+                pCellRendererCombo->set_use_second(use_second);
+
+                LayoutItem_Field layout_field_second;
+                if(use_second)
+                {
+                  Document_Glom* document = get_document();
+                  if(document)
+                  {
+                    Field field_second; //TODO: Actually show this in the combo:
+                    document->get_field(to_table, choice_second, field_second);
+
+                    layout_field_second.m_field = field_second;
+
+                    //We use the default formatting for this field.
+                  }
+                }
+
+                type_list_values_with_second list_values = get_choice_values(column_info.m_field);
+                for(type_list_values_with_second::const_iterator iter = list_values.begin(); iter != list_values.end(); ++iter)
+                {
+                  const Glib::ustring first = GlomConversions::get_text_for_gda_value(column_info.m_field.m_field.get_glom_type(), iter->first, column_info.m_field.m_numeric_format);
+
+                  Glib::ustring second;
+                  if(use_second)
+                    second = GlomConversions::get_text_for_gda_value(layout_field_second.m_field.get_glom_type(), iter->second, layout_field_second.m_numeric_format);
+
+                  pCellRendererCombo->append_list_item(first, second);
+                }
+              }
+            }
           }
         }
         else
@@ -826,7 +849,7 @@ guint DbAddDel::add_column(const LayoutItem_Field& field)
   //column_info.m_visible = visible;
 
   //Make it non-editable if it is auto-generated:
-  if(field.m_field.get_field_info().get_auto_increment())
+  if(field.m_field.get_auto_increment())
     column_info.m_editable = false;
   else
     column_info.m_editable = field.get_editable_and_allowed();
@@ -1180,7 +1203,7 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
       if(!success)
       {
           //Tell the user and offer to revert or try again:
-          bool revert = glom_show_dialog_invalid_date(field_type);
+          bool revert = glom_show_dialog_invalid_data(field_type);
           if(revert)
           {
             //Revert the data:
